==> OpenCL-0.8.5.tgz: OpenCL-0.8.5/src/dsa.cpp <==
/*************************************************
* DSA Source File                                *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/dsa.h>
#include <opencl/numthry.h>
#include <opencl/primes.h>

#include <stdio.h>

namespace OpenCL {

/*************************************************
* DSA_PublicKey Constructor                      *
*************************************************/
DSA_PublicKey::DSA_PublicKey(const BigInt& prime, const BigInt& subgroup,
                             const BigInt& generator, const BigInt& y1) :
   p(prime), q(subgroup), g(generator), y(y1),
   powermod_g_p(g, p), powermod_y_p(y, p)
   {
   printf("WARNING: This constructor of DSA_PublicKey is deprecated; "
          "(read doc/deprecated.txt for details)\n");

   if(p <= 7 || q <= 3 || q >= p || g <= 1 || g >= p)
      throw Invalid_Argument("DSA_PublicKey: Invalid prime or generator");
   if(y < 0 || y >= p)
      throw Invalid_Argument("DSA_PublicKey: Invalid public parameter");
   }

/*************************************************
* DSA_PublicKey Constructor                      *
*************************************************/
DSA_PublicKey::DSA_PublicKey(const DL_Group& domain, const BigInt& key) :
   p(domain.get_p()), q(domain.get_q()), g(domain.get_g()), y(key),
   powermod_g_p(g, p), powermod_y_p(y, p)
   {
   if(p <= 7 || q <= 3 || q >= p || g <= 1 || g >= p)
      throw Invalid_Argument("DSA_PublicKey: Invalid prime or generator");
   if(y < 0 || y >= p)
      throw Invalid_Argument("DSA_PublicKey: Invalid public parameter");
   }

/*************************************************
* Check Public DSA Parameters                    *
*************************************************/
bool DSA_PublicKey::check_params() const
   {
   if((p - 1) % q != 0)
      return false;
   if(y <= 1 || y >= p)
      return false;
   if(!verify_prime(p) || !verify_prime(q))
      return false;
   return true;
   }

/*************************************************
* DSA Verification Function                      *
*************************************************/
bool DSA_PublicKey::verify(const byte msg[], u32bit msg_len,
                           const byte sig[], u32bit sig_len) const
   {
   if(sig_len != 2*q.bytes() || msg_len != q.bytes())
      return false;

   t1.binary_decode(msg, msg_len);
   r.binary_decode(sig, sig_len / 2);
   s.binary_decode(sig + sig_len / 2, sig_len / 2);

   if(r.is_zero() || s.is_zero() || r >= q || s >= q)
      return false;

   BigInt w = inverse_mod(s, q);
   BigInt u1 = (w * t1) % q;
   BigInt u2 = (w * r) % q;
   BigInt v = (powermod_g_p(u1) * powermod_y_p(u2)) % p;
   return (v % q == r);
   }

/*************************************************
* Create a DSA Key                               *
*************************************************/
DSA_PrivateKey::DSA_PrivateKey(u32bit pbits, bool use_fips_keygen)
   {
   printf("WARNING: This constructor of DSA_PrivateKey is deprecated; "
          "(read doc/deprecated.txt for details)\n");

   if(pbits < 512 || pbits % 8)
      throw Invalid_Argument("DSA_PrivateKey: Invalid size " +
                             to_string(pbits) + " for prime p");

   if(use_fips_keygen && (!valid_fips186_p_size(pbits)))
      throw Invalid_Argument("DSA_PrivateKey: Size of p not legal for "
                             "FIPS-186 prime generation");

   if(use_fips_keygen)
      generate_dsa_primes(p, q, pbits);
   else
      {
      u32bit qbits = 2 * dl_work_factor(pbits);
      q = random_prime(qbits);
      BigInt q_2 = 2 * q;
      do
         p = random_integer(pbits - qbits - 1) * q_2 + 1;
      while(p.bits() != pbits || !is_prime(p));
      }

   keygen_operation();
   }

/*************************************************
* DSA_PrivateKey Constructor                     *
*************************************************/
DSA_PrivateKey::DSA_PrivateKey(const BigInt& prime, const BigInt& subgroup,
                               const BigInt& generator) :
   DSA_PublicKey(prime, subgroup, generator, 0)
   {
   printf("WARNING: This constructor of DSA_PrivateKey is deprecated; "
          "(read doc/deprecated.txt for details)\n");

   x = random_integer(1, q - 1);
   y = powermod_g_p(x);
   powermod_y_p = FixedBase_Exp(y, p);
   }

/*************************************************
* DSA_PrivateKey Constructor                     *
*************************************************/
DSA_PrivateKey::DSA_PrivateKey(const BigInt& prime, const BigInt& subgroup,
                               const BigInt& generator, const BigInt& x1,
                               const BigInt& y1) :
   DSA_PublicKey(prime, subgroup, generator, y1), x(x1)
   {
   printf("WARNING: This constructor of DSA_PrivateKey is deprecated; "
          "(read doc/deprecated.txt for details)\n");

   if(x <= 1 || y <= 1 || x >= p || y >= p)
      throw Invalid_Argument("DSA_PrivateKey: invalid private key");
   }

/*************************************************
* Create a DSA Key from a seed                   *
*************************************************/
DSA_PrivateKey::DSA_PrivateKey(const byte seed[], u32bit seed_length,
                               u32bit pbits, u32bit start)
   {
   printf("WARNING: This constructor of DSA_PrivateKey is deprecated; "
          "(read doc/deprecated.txt for details)\n");

   if(!generate_dsa_primes(p, q, seed, seed_length, pbits, start))
      throw Invalid_Argument("DSA_PrivateKey: The seed value given does not "
                             "generate a DSA key");
   keygen_operation();
   }

/*************************************************
* DSA_PrivateKey Constructor                     *
*************************************************/
DSA_PrivateKey::DSA_PrivateKey(const DL_Group& domain) :
   DSA_PublicKey(domain, 0)
   {
   x = random_integer(1, q - 1);
   y = powermod_g_p(x);
   powermod_y_p = FixedBase_Exp(y, p);
   }

/*************************************************
* DSA_PrivateKey Constructor                     *
*************************************************/
DSA_PrivateKey::DSA_PrivateKey(const DL_Group& domain,
                               const BigInt& priv, const BigInt& pub) :
   DSA_PublicKey(domain, pub), x(priv)
   {
   if(x <= 1 || x >= p)
      throw Invalid_Argument("DSA_PrivateKey: Invalid private parameter");
   }

/*************************************************
* Handle Parts of DSA Key Generation             *
*************************************************/
void DSA_PrivateKey::keygen_operation()
   {
   BigInt e = (p - 1) / q;
   u32bit prime_no = 0;
   do
      g = power_mod(PRIMES[prime_no++], e, p);
   while(g == 1);
   powermod_g_p = FixedBase_Exp(g, p);

   x = random_integer(q.bits() - 1);
   y = powermod_g_p(x);
   powermod_y_p = FixedBase_Exp(y, p);
   }

/*************************************************
* Check Private DSA Parameters                   *
*************************************************/
bool DSA_PrivateKey::check_params() const
   {
   if(!DSA_PublicKey::check_params())
      return false;
   if(y != powermod_g_p(x))
      return false;
   return true;
   }

/*************************************************
* Check for a p size authorized by FIPS 186-2    *
*************************************************/
bool DSA_PrivateKey::valid_fips186_p_size(u32bit pbits) const
   {
   return ((pbits % 64 == 0) || (pbits >= 512) || (pbits <= 1024));
   }

/*************************************************
* DSA Signature Operation                        *
*************************************************/
SecureVector<byte> DSA_PrivateKey::sign(const byte in[], u32bit len) const
   {
   t1.binary_decode(in, len);

   while(true)
      {
      BigInt k(Random, q.bits());
      while(k >= q)
         k.randomize(q.bits());

      r = powermod_g_p(k) % q;
      s = (inverse_mod(k, q) * mul_add(x, r, t1)) % q;

      if(r.is_zero() || s.is_zero())
         continue;

      SecureVector<byte> output(2*q.bytes());
      r.binary_encode(output + (q.bytes() - r.bytes()));
      s.binary_encode(output + output.size() / 2 + (q.bytes() - s.bytes()));
      return output;
      }
   }

}

==> OpenCL-0.9.0.tgz: OpenCL-0.9.0/src/dsa.cpp <==
/*************************************************
* DSA Source File                                *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/dsa.h>
#include <opencl/numthry.h>
#include <opencl/primes.h>

namespace OpenCL {

/*************************************************
* DSA_PublicKey Constructor                      *
*************************************************/
DSA_PublicKey::DSA_PublicKey(const DL_Group& domain, const BigInt& key) :
   group(domain), p(group.get_p()), q(group.get_q()), g(group.get_g()),
   y(key), powermod_g_p(g, p), powermod_y_p(y, p)
   {
   if(y < 0 || y >= p)
      throw Invalid_Argument("DSA_PublicKey: Invalid public parameter");
   }

/*************************************************
* DSA_PublicKey Copy Constructor                 *
*************************************************/
DSA_PublicKey::DSA_PublicKey(const DSA_PublicKey& key) :
   PK_Verifying_wo_MR_Key(),
   group(key.group), p(group.get_p()), q(group.get_q()), g(group.get_g()),
   y(key.get_y()), powermod_g_p(g, p), powermod_y_p(y, p)
   {
   }

/*************************************************
* Check Public DSA Parameters                    *
*************************************************/
bool DSA_PublicKey::check_params() const
   {
   if(y < 2 || y >= p)
      return false;
   if(!group.verify())
      return false;
   return true;
   }

/*************************************************
* DSA Verification Function                      *
*************************************************/
bool DSA_PublicKey::verify(const byte msg[], u32bit msg_len,
                           const byte sig[], u32bit sig_len) const
   {
   if(sig_len != 2*q.bytes() || msg_len > q.bytes())
      return false;

   BigInt t1(msg, msg_len);
   BigInt r(sig, q.bytes());
   BigInt s(sig + q.bytes(), q.bytes());

   if(r.is_zero() || s.is_zero() || r >= q || s >= q)
      return false;

   BigInt w = inverse_mod(s, q);
   BigInt u1 = (w * t1) % q;
   BigInt u2 = (w * r) % q;
   BigInt v = (powermod_g_p(u1) * powermod_y_p(u2)) % p;
   return (v % q == r);
   }

/*************************************************
* DSA_PrivateKey Constructor                     *
*************************************************/
DSA_PrivateKey::DSA_PrivateKey(const DL_Group& domain) :
   DSA_PublicKey(domain, 0)
   {
   x = random_integer(1, q - 1);
   y = powermod_g_p(x);
   powermod_y_p = FixedBase_Exp(y, p);
   }

/*************************************************
* DSA_PrivateKey Constructor                     *
*************************************************/
DSA_PrivateKey::DSA_PrivateKey(const DL_Group& domain,
                               const BigInt& priv, const BigInt& pub) :
   DSA_PublicKey(domain, pub), x(priv)
   {
   if(x <= 1 || x >= p)
      throw Invalid_Argument("DSA_PrivateKey: Invalid private parameter");
   }

/*************************************************
* Check Private DSA Parameters                   *
*************************************************/
bool DSA_PrivateKey::check_params() const
   {
   if(!DSA_PublicKey::check_params())
      return false;
   if(y != powermod_g_p(x))
      return false;
   return true;
   }

/*************************************************
* DSA Signature Operation                        *
*************************************************/
SecureVector<byte> DSA_PrivateKey::sign(const byte in[], u32bit len) const
   {
   BigInt t1(in, len);

   while(true)
      {
      BigInt k(Random, q.bits());
      while(k >= q)
         k.randomize(q.bits());

      BigInt r = powermod_g_p(k) % q;
      BigInt s = (inverse_mod(k, q) * mul_add(x, r, t1)) % q;

      if(r.is_zero() || s.is_zero())
         continue;

      SecureVector<byte> output(2*q.bytes());
      r.binary_encode(output + (q.bytes() - r.bytes()));
      s.binary_encode(output + output.size() / 2 + (q.bytes() - s.bytes()));
      return output;
      }
   }

}
